import processing.opengl.*;
import peasy.*;
PVector v;
PFont font;
int numberOfVertices = 16;
int numberOfRows = 12;
float mapWidth = 200.0;
float mapDepth = 400.0;
float mapHeightStep = 10.0;
PeasyCam cam;
boolean spreadSwitch = false;


void setup() {
  size(1280, 720, OPENGL);
  colorMode(HSB, 360, 100, 100, 100);
  arrayAssignment();
  importSQL();
  //camera settings
  cam = new PeasyCam(this, 300, 0, -600, 1200);
  cam.setMinimumDistance(10);
  cam.setMaximumDistance(1600);

  //font = createFont("Helvetica", 20); 
  font = createFont("Futura-Medium", 20);
} 

void draw()
{
  background(0);
  noFill();
  pushMatrix();
  translate(width/2-(mapWidth*2.2), height/2-(mapDepth*.4), -50);
  rotateX(-PI/2);
  //rotateX(setRotation.x);
  //rotateY(setRotation.y);
  //rotateZ(setRotation.z);

  drawMapLine();
  drawMapOuter();
  //drawCircles();
  drawSectionLabel();
  drawSections();
  popMatrix();

  cam.beginHUD();
  noStroke();
  fill(0, 100, 0, 100);
  //rect(15, 10, 200, 50);
  rect(0, 0, width, 60);
  rect(0, 0, 60, height);
  rect(0, height-60, width, 60);
  rect(width-60, 0, 60, height);
  stroke(0, 0, 100, 50);
  strokeWeight(.1);
  noFill();
  rect(60, 60, width-120, height-120);

  textAlign(LEFT, CENTER);
  fill(0, 0, 100, 100);
  textFont(font, 12); 
  text("Seattle Public Library: Dewey Map", 60, 40);
  text("2011.01.01 - 2011.02.01", width/2-70, 40);
  if ((keyTime == 0)||(keyTime == 1)||(keyTime == 2)) 
  {
    text("Time = 0" + (keyTime + 7) + ":00", width-130, 40);
  }
  else
  {
    text("Time = " + (keyTime + 7) + ":00", width-130, 40);
  }
  cam.endHUD();
}


void drawMapLine()
{
  v = new PVector(0.0, 0.0, 0.0);
  noFill();
  smooth();
  stroke(0, 80, 100, 50);
  strokeWeight(1);
  strokeJoin(MITER);
  beginShape();
  for (int i = 0; i< numberOfVertices; i++)
  {
    if (spreadSwitch == true) {
      vertex(v.x, v.y, v.z);
      if (i% (numberOfVertices/4) ==0) v.z-= mapDepth;
      else if (i% (numberOfVertices/4) ==1) v.x+= mapWidth+i*35;
      else if (i% (numberOfVertices/4) ==2) v.z+= mapDepth;
      else if (i% (numberOfVertices/4) ==3) v.x-= mapWidth+i*35;
      v.y-=mapHeightStep;
    }
    else {
      vertex(v.x, v.y, v.z);
      if (i% (numberOfVertices/4) ==0) v.z-= mapDepth;
      else if (i% (numberOfVertices/4) ==1) v.x+= mapWidth;
      else if (i% (numberOfVertices/4) ==2) v.z+= mapDepth;
      else if (i% (numberOfVertices/4) ==3) v.x-= mapWidth;
      v.y-=mapHeightStep;
    }
  }
  endShape();
}

void drawMapOuter()
{
  stroke(0, 0, 100, 50);
  strokeWeight(1);
  strokeJoin(MITER);
  pushMatrix();
  scale(1.1);
  translate(-110, 0, 55);
  for (int i=0;i< 1;i++)
  {
    beginShape();
    vertex (0, i*-5, 0);
    vertex (0, i*-5, -400);
    vertex (20, i*-5, -440);
    vertex (20, i*-5, -490);
    vertex(100, i*-5, -490);
    vertex(400, i*-5, -460);
    vertex(390, i*-5, -80);
    vertex(260, i*-5, 0);
    endShape(CLOSE);
  }  
  popMatrix();
}

void drawSections()
{  
  smooth();
  //noFill();
  //noStroke();
  //stroke(255,255,255,80);
  //fill(255, 255, 255, 125);
  stroke(0, 0, 100, 40);
  strokeWeight(.01);
  for (int i=0;i<96;i++)
  {
    //for (int j=0;j<16;j++)
    //{
    pushMatrix();
    //if the spreadswitch is on, we're going to spread things out as we go up. 
    if (spreadSwitch == true) {
      if (int(i/12)%2 == 0) {
        translate(deweyClassPosition[i].x-(int(i/12)*35), deweyClassPosition[i].y, deweyClassPosition[i].z);//left
      }
      else {
        //translate(deweyClassPosition[i].x+(210-(int(i/12)*30)), deweyClassPosition[i].y, deweyClassPosition[i].z);//right
        translate(deweyClassPosition[i].x+(int(i/12)*35), deweyClassPosition[i].y, deweyClassPosition[i].z);
      }
    }
    else {
      translate(deweyClassPosition[i].x, deweyClassPosition[i].y, deweyClassPosition[i].z);
    }
    //fill (0,100,100,(i*100/96));
    //fill (255,255,255,(deweyClassPosition[i].z*255/-400));
    //fill (data[9][i]*360/20068,100,100,100);
    //fill (keyTime*360/16, 80, 100, (data[keyTime][i]*100/data[keyTime][78]));
    fill (0, 80, 100, (data[keyTime][i]*100/data[keyTime][78]));
    box(60, 10, 30);
    popMatrix();
  }
}

void drawSectionLabel()
{  
  smooth();
  //noFill();
  //noStroke();
  //stroke(255,255,255,80);
  //fill(255, 255, 255, 125);
  stroke(0, 0, 100, 50);
  strokeWeight(.1);
  for (int i=0;i<10;i++)
  {
    //for (int j=0;j<16;j++)
    //{
    pushMatrix();
    translate(deweyClassPosition[i].x-50, deweyClassPosition[i].y, deweyClassPosition[i].z);
    rotateX(PI/2);
    fill(0, 0, 100, 100);
    textFont(font, 8); 
    text("0"+i+"0", 0, 0);
    translate(0, 0, -1);
    fill(0, 100, 0, 100);
    rect(0, -15, 20, 20);
    popMatrix();
    //println(deweyClassPosition[i].z);
    //}
  }

  for (int i=10;i<96;i++)
  {
    //for (int j=0;j<16;j++)
    //{
    pushMatrix();
    //if the spreadswitch is on, we're going to spread things out as we go up. 
    if (spreadSwitch == true) {
      if (int(i/12)%2 == 0) {
        translate(deweyClassPosition[i].x-(int(i/12)*35)-50, deweyClassPosition[i].y, deweyClassPosition[i].z);//left
      }
      else {
        //translate(deweyClassPosition[i].x+(210-(int(i/12)*30))+30, deweyClassPosition[i].y, deweyClassPosition[i].z);
        translate(deweyClassPosition[i].x+(int(i/12)*35)+30, deweyClassPosition[i].y, deweyClassPosition[i].z);//right
      }
    }
    else {
      //DRAW labels on the right side if we're on the right side of the library
      if (int(i/12)%2 == 0) {
        translate(deweyClassPosition[i].x-50, deweyClassPosition[i].y, deweyClassPosition[i].z);
      }
      else {
        translate(deweyClassPosition[i].x+30, deweyClassPosition[i].y, deweyClassPosition[i].z);
      }
    }    


    //translate(deweyClassPosition[i].x-50, deweyClassPosition[i].y, deweyClassPosition[i].z);
    rotateX(PI/2);
    fill(0, 0, 100, 100);
    textFont(font, 8); 
    text(i+"0", 0, 0);
    translate(0, 0, -1);
    fill(0, 100, 0, 100);
    rect(0, -15, 20, 20);
    popMatrix();
    //println(deweyClassPosition[i].z);
    //}
  }
}



/*
void drawCircles()
 {
 fill(255, 0, 0, 255);
 for (int i=0;i<96;i++)
 {
 float ellipseXPos = random (-30,30);
 pushMatrix();
 translate(deweyClassPosition[i].x, deweyClassPosition[i].y, deweyClassPosition[i].z);
 for (int j=0;j<1;j++)
 {
 ellipse(ellipseXPos, 0, 5, 5);
 }
 popMatrix();
 }
 pushMatrix();
 fill(0, 255, 0, 255);
 translate(deweyClassPosition[0].x, deweyClassPosition[0].y, deweyClassPosition[0].z);
 ellipse(0, 0, 5, 5);
 popMatrix();
 }
 */
